﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TribesPSO
{
    /// <summary>
    /// A single threaded implementation of search space that uses HypersphereParticles
    /// </summary>
    public sealed class SingleThreadedHypersphereSearchSpace : SingleThreadedSearchSpace<HypersphereParticle>
    {
        /// <summary>
        /// Creates a new single threaded hypersphere search space to optimize the specified objective function
        /// </summary>
        /// <param name="objectiveFunction">The IObjectiveFunction to solve</param>
        public SingleThreadedHypersphereSearchSpace(IObjectiveFunction objectiveFunction)
            : base(objectiveFunction) { }

        /// <summary>
        /// Creates a new single threaded hypersphere search space to optimize the specified objective function.
        /// The specified random number generator will be used to move the particles
        /// </summary>
        /// <param name="objectiveFunction">The IObjectiveFunction to solve</param>
        /// <param name="randomNumberGenerator">The random number generator that will be used to move the particles</param>
        public SingleThreadedHypersphereSearchSpace(IObjectiveFunction objectiveFunction, IHyperspaceRandom randomNumberGenerator)
            : base(objectiveFunction,randomNumberGenerator) { }

        /// <summary>
        /// Creates a new HypersphereParticle at the specified location and the SearchSpaces own IObjectiveFunction
        /// </summary>
        /// <param name="position">The initial location of the new particle</param>
        /// <returns>A new IndependentGaussianParticle</returns>
        protected override HypersphereParticle GenerateParticleAtPosition(EuclidianVector position)
        {
            return new HypersphereParticle(this.GoodnessFunction, position);
        }
    }
}
